import pytest
import logging
import runregistry

logger = logging.getLogger(__name__)
VALID_RUN_NUMBER = 362874
VALID_DATASET_NAME = "/PromptReco/Commissioning2021/DQM"


@pytest.fixture
def setup_runregistry():
    logger.info("Connecting to development runregistry")
    runregistry.setup("development")


def test_move_datasets(setup_runregistry):
    answers = runregistry.move_datasets(
        from_=runregistry.WAITING_DQM_GUI_CONSTANT,
        to_="OPEN",
        dataset_name=VALID_DATASET_NAME,
        run=VALID_RUN_NUMBER,
        workspace="global",
    )
    # TODO: Run also with a token that has permission
    assert all([answer.status_code == 401 for answer in answers])
    answers = runregistry.move_datasets(
        from_="OPEN",
        to_="SIGNOFF",
        dataset_name=VALID_DATASET_NAME,
        run=VALID_RUN_NUMBER,
        workspace="ctpps",
    )
    # Requires permission
    assert all([answer.status_code == 401 for answer in answers])


def test_make_significant_single_run(setup_runregistry):
    # Get latest run in dev runregistry and make it significant
    run = runregistry.get_runs(limit=1, filter={})[0]
    answer = runregistry.make_significant_runs(run=run["run_number"])
    # requires permission
    assert answer.status_code == 401


def test_make_significant_multi_runs(setup_runregistry):
    # Get latest run in dev runregistry and make it significant
    run = runregistry.get_runs(limit=1, filter={})[0]
    answers = runregistry.make_significant_runs(runs=[run["run_number"]])
    # requires permission
    assert all([answer.status_code == 401 for answer in answers])


def test_reset_RR_attributes_and_refresh_runs_signed_off(setup_runregistry):
    answers = runregistry.reset_RR_attributes_and_refresh_runs(runs=VALID_RUN_NUMBER)
    # Cannot refresh runs which are not open
    assert all(
        [
            answer.status_code == 500 and "Run must be in state OPEN" in answer.text
            for answer in answers
        ]
    )


def test_manually_refresh_components_statuses_for_runs_open(setup_runregistry):
    run = runregistry.get_runs(limit=1, filter={})[0]
    answers = runregistry.manually_refresh_components_statuses_for_runs(
        runs=run["run_number"]
    )
    assert all([answer.status_code == 200 for answer in answers])


def test_reset_RR_attributes_and_refresh_runs_open(setup_runregistry):
    run = runregistry.get_runs(limit=1, filter={})[0]
    answers = runregistry.reset_RR_attributes_and_refresh_runs(runs=run["run_number"])
    assert all([answer.status_code == 200 for answer in answers])


def test_manually_refresh_components_statuses_for_runs_signed_off(setup_runregistry):
    answers = runregistry.manually_refresh_components_statuses_for_runs(
        runs=VALID_RUN_NUMBER
    )
    # Cannot refresh runs which are not open
    assert all(
        [
            answer.status_code == 500 and "Run must be in state OPEN" in answer.text
            for answer in answers
        ]
    )


def test_move_runs_no_run_arg(setup_runregistry):
    with pytest.raises(ValueError):
        # Raises ValueError
        runregistry.move_runs("OPEN", "SIGNOFF")


def test_move_single_run(setup_runregistry):
    """
    Unfortunately, this function was given a dual signature, and can return
    both a single or a list of request responses.
    """
    answer = runregistry.move_runs("OPEN", "SIGNOFF", run=VALID_RUN_NUMBER)
    # Requires permission
    assert answer.status_code == 401


def test_move_multi_runs(setup_runregistry):
    """
    Unfortunately, this function was given a dual signature, and can return
    both a single or a list of request responses.
    """
    answers = runregistry.move_runs("OPEN", "SIGNOFF", runs=[VALID_RUN_NUMBER])
    # Requires permission
    assert all([answer.status_code == 401 for answer in answers])


def test_edit_rr_lumisections(setup_runregistry):
    answer = runregistry.edit_rr_lumisections(
        VALID_RUN_NUMBER, 0, 1, "castor-castor", "GOOD"
    )
    # Requires permission
    assert answer.status_code == 401



from runregistry.utils import transform_to_rr_run_filter


class TestFilterCreation:
    def test_get_run(self):
        run_number = 323434
        assert transform_to_rr_run_filter(run_filter={"run_number": run_number}) == {
            "run_number": {"=": run_number}
        }

    def test_get_multiple_run_using_or(self):
        run_number1 = 323555
        run_number2 = 323444
        run_number3 = 343222
        run_number4 = 333333
        user_input = {
            "run_number": {
                "or": [run_number1, run_number2, run_number3, {"=": run_number4}]
            }
        }
        desired_output = {
            "run_number": {
                "or": [
                    {"=": run_number1},
                    {"=": run_number2},
                    {"=": run_number3},
                    {"=": run_number4},
                ]
            }
        }

        assert transform_to_rr_run_filter(run_filter=user_input) == desired_output



# run attributes:
run_table_attributes = "run_number"

run_triplet_attributes = (
    "castor-castor",
    "cms-cms",
    "csc-csc",
    "ctpps-ctpps",
    "dt-dt",
    "ecal-ecal",
    "ecal-es",
    "hcal-hcal",
    "hlt-hlt",
    "l1t-l1t",
    "l1t-l1tcalo",
    "l1t-l1tmu",
    "lumi-lumi",
    "rpc-rpc",
    "tracker-pixel",
    "tracker-strip",
    # Non global:
    "btag-btag",
    "csc-mep",
    "csc-mem",
    "csc-triggerpe",
    "csc-csctf",
    "csc-ddus",
    "csc-efficiency",
    "csc-gasgains",
    "csc-integrity",
    "csc-timing",
    "csc-strips",
    "csc-resolution",
    "csc-segments",
    "csc-pedestals",
    "csc-occupancy",
    "ctpps-rp56_220",
    "ctpps-rp56_cyl",
    "ctpps-rp56_210",
    "ctpps-rp45_cyl",
    "ctpps-rp45_220",
    "ctpps-rp45_210",
    "ecal-tpg",
    "ecal-timing",
    "ecal-preshower",
    "ecal-noise",
    "ecal-laser",
    "ecal-esp",
    "ecal-esm",
    "ecal-eep",
    "ecal-eem",
    "ecal-ebp",
    "ecal-ebm",
    "ecal-collisions",
    "ecal-analysis",
    "egamma-egamma",
    "egamma-es",
    "hcal-hb",
    "hcal-ho12ls",
    "hcal-ho0ls",
    "hcal-hfls",
    "hcal-hels",
    "hcal-hbls",
    "hcal-ho12",
    "hcal-ho0",
    "hcal-hf",
    "hcal-he",
    "hlt-muons",
    "hlt-electrons",
    "hlt-photons",
    "hlt-jetmet",
    "hlt-tau",
    "hlt-bjets",
    "hlt-technical",
    "hlt-global",
    "jetmet-jetmet",
    "l1t-bptx_tech",
    "l1t-bcs_tech",
    "l1t-hf_tech",
    "l1t-rpc_tech",
    "l1t-software",
    "l1t-muon",
    "l1t-muon_dt",
    "l1t-muon_rpc",
    "l1t-muon_csc",
    "l1t-e_gamma",
    "l1t-jet",
    "l1t-energy_sums",
    "l1t-hf_rings" "rpc-elog",
    "rpc-lv",
    "rpc-hv",
    "rpc-feb",
    "rpc-noise",
    "rpc-strip",
    "tau-tau",
)

run_rr_attributes = (
    "class",
    "state",
    "significant",
    "stop_reason",
    "short_run",
    "run_needs_to_be_updated_manually",
)

run_oms_attributes = (
    "energy",
    "l1_key",
    "b_field",
    "hlt_key",
    "l1_menu",
    "l1_rate",
    "end_time",
    "sequence",
    "duration",
    "end_lumi",
    "init_lumi",
    "clock_type",
    "components",
    "run_number",
    "start_time",
    "fill_number",
    "l1_hlt_mode",
    "last_update",
    "ls_duration",
    "stable_beam",
    "hf_included",
    "dt_included",
    "es_included",
    "csc_included",
    "cow_included",
    "daq_included",
    "dcs_included",
    "dqm_included",
    "gem_included",
    "ltc_included",
    "rpc_included",
    "trg_included",
    "trigger_mode",
    "hcal_included",
    "scal_included",
    "efed_included",
    "recorded_lumi",
    "ecal_included",
    "lumi_included",
    "tcds_included",
    "cmssw_version",
    "delivered_lumi",
    "totem_included",
    "tier0_transfer",
    "pixel_included",
    "ctpps_included",
    "hflumi_included",
    "l1_key_stripped",
    "castor_included",
    "hlt_physics_size",
    "fill_type_party2",
    "tracker_included",
    "dt_efed_included",
    "fill_type_party1",
    "injection_scheme",
    "hlt_physics_rate",
    "rpc_efed_included",
    "trg_efed_included",
    "fill_type_runtime",
    "pixel_up_included",
    "csc_efed_included",
    "ctpps_tot_included",
    "ecal_efed_included",
    "hcal_efed_included",
    "l1_triggers_counter",
    "hlt_physics_counter",
    "l1_hlt_mode_stripped",
    "tracker_efed_included",
    "initial_prescale_index",
    "hlt_physics_throughput",
    "beams_present_and_stable",
)

# dataset attributes:
dataset_table_attributes = ("run_number", "dataset_name")

dataset_attributes = (
    "appeared_in",
    "global_state",
    "dt_state",
    "cms_state",
    "csc_state",
    "hlt_state",
    "l1t_state",
    "rpc_state",
    "tau_state",
    "btag_state",
    "ecal_state",
    "hcal_state",
    "lumi_state",
    "muon_state",
    "ctpps_state",
    "castor_state",
    "egamma_state",
    "jetmet_state",
    "tracker_state",
)

dataset_triplet_attributes = (
    "dt-dt",
    "csc-tf",
    "rpc-hv",
    "rpc-lv",
    "cms-cms",
    "csc-csc",
    "ecal-es",
    "hcal-hb",
    "hcal-he",
    "hcal-hf",
    "hlt-hlt",
    "hlt-tau",
    "l1t-l1t",
    "rpc-feb",
    "rpc-rpc",
    "tau-tau",
    "ecal-ebm",
    "ecal-ebp",
    "ecal-eem",
    "ecal-eep",
    "ecal-esm",
    "ecal-esp",
    "ecal-tpg",
    "hcal-ho0",
    "rpc-elog",
    "btag-btag",
    "ecal-ecal",
    "hcal-hcal",
    "hcal-ho12",
    "hlt-bjets",
    "hlt-muons",
    "l1t-l1tmu",
    "lumi-lumi",
    "muon-muon",
    "rpc-noise",
    "csc-strips",
    "csc-timing",
    "ecal-laser",
    "ecal-noise",
    "hcal-hb_ls",
    "hcal-he_ls",
    "hcal-hf_ls",
    "hlt-jetmet",
    "ctpps-ctpps",
    "ecal-timing",
    "hcal-hoO_ls",
    "hlt-photons",
    "l1t-e_gamma",
    "l1t-hf_tech",
    "l1t-l1tcalo",
    "l1t-muon_dt",
    "tracker-pix",
    "csc-gasgains",
    "csc-segments",
    "hcal-ho12_ls",
    "l1t-bcs_tech",
    "l1t-hg_rings",
    "l1t-muon_csc",
    "l1t-muon_rpc",
    "l1t-rpc_tech",
    "l1t-software",
    "castor-castor",
    "csc-integrity",
    "csc-occupancy",
    "csc-pedestals",
    "ecal-analysis",
    "egamma-egamma",
    "hlt-electrons",
    "hlt-technical",
    "jetmet-jetmet",
    "l1t-bptx_tech",
    "tracker-strip",
    "tracker-track",
    "csc-efficiency",
    "csc-resolution",
    "csc-triggergpe",
    "ctpps-rp45_220",
    "ctpps-rp45_cyl",
    "ctpps-rp56_210",
    "ctpps-rp56_220",
    "ctpps-rp56_cyl",
    "ecal-preshower",
    "hlt-hlt_global",
    "ecal-collisions",
    "l1t-energy_sums",
    "cms-infrastructure",
)



"""
Very simple stress test to make sure that the proxy can handle
lots of requests without being too slow for the users via the
browser.

Increase the number of pods on the proxy by right-clicking
it under Developer --> Topology and then "Edit Pod count".

On PaaS, you can monitor performance by going to

Developer --> Observe

and change Time Range to 5 minutes and Refresh Interval to 15 seconds.

This test is not intended to be run by pytest, just during development.
"""

import concurrent.futures
import runregistry

runregistry.setup("development")

NUM_THREADS = 10
NUM_JOBS = 5000


def fetch_run_data():
    a = runregistry.get_run(355555)


if __name__ == "__main__":
    # Run the same request up to a total of NUM_JOBS times,
    # using a ThreadPoolExecutor to do the requests concurrently.
    executor = concurrent.futures.ThreadPoolExecutor(max_workers=NUM_THREADS)
    for i in range(NUM_JOBS):
        executor.submit(fetch_run_data)



import json
import warnings

from runregistry.runregistry import (
    get_run,
    get_runs,
    get_dataset_names_of_run,
    get_dataset,
    get_datasets,
    get_lumisections,
    get_oms_lumisections,
    get_lumisection_ranges,
    get_oms_lumisection_ranges,
    get_joint_lumisection_ranges,
    generate_json,
    create_json,
    setup,
)

VALID_RUN_NUMBER = 327743
VALID_RUN_RANGE_START = 309000
VALID_RUN_RANGE_STOP = 310000

INVALID_RUN_NUMBER = 420420420
VALID_DATASET_NAME = "/PromptReco/HICosmics18A/DQM"
INVALID_DATASET_NAME = "/PromptKikiriko/HELLOCosmics18Z/DQM"


def test_get_run():
    run_number = VALID_RUN_NUMBER
    run = get_run(run_number=VALID_RUN_NUMBER)
    assert run["run_number"] == VALID_RUN_NUMBER
    # Non-existent run
    run_number = INVALID_RUN_NUMBER
    run = get_run(run_number=run_number)
    assert not run


def test_get_runs():
    # Gets runs between run number VALID_RUN_RANGE_START and VALID_RUN_RANGE_STOP
    filter_run = {
        "run_number": {
            "and": [{">": VALID_RUN_RANGE_START}, {"<": VALID_RUN_RANGE_STOP}]
        }
    }
    runs = get_runs(filter=filter_run)
    assert len(runs) > 0
    # Gets runs that contain lumisections that classified DT as GOOD AND lumsiections that classified hcal as STANDBY
    filter_run = {
        "run_number": {
            "and": [{">": VALID_RUN_RANGE_START}, {"<": VALID_RUN_RANGE_STOP}]
        },
        "dt-dt": "GOOD",
        # 'hcal': 'STANDBY'
    }

    runs = get_runs(filter=filter_run)
    assert len(runs) > 0
    runs = []

    filter_run = {
        "run_number": {
            "and": [{">": VALID_RUN_RANGE_START}, {"<": VALID_RUN_RANGE_STOP}]
        },
        "tracker-strip": "GOOD",
    }
    runs = get_runs(filter=filter_run)
    print(json.dumps(runs))
    assert len(runs) > 0


def test_get_runs_with_ignore_filter():
    filter_run = {
        "run_number": {
            "and": [{">": VALID_RUN_RANGE_START}, {"<": VALID_RUN_RANGE_STOP}]
        },
        "oms_attributes.hlt_key": {"like": "%commissioning2018%"},
        "triplet_summary.dt-dt.GOOD": {">": 0},
    }
    runs = get_runs(filter=filter_run, ignore_filter_transformation=True)
    assert len(runs) > 0


def test_get_datasets_with_ignore_filter():
    # datasets = get_datasets(filter={
    #     "run_number": {
    #         "and": [{
    #             ">": VALID_RUN_RANGE_START
    #         }, {
    #             "<": VALID_RUN_RANGE_STOP
    #         }]
    #     },
    #     "oms_attributes.hlt_key": {
    #         "like": "%commissioning2018%"
    #     },
    #     "triplet_summary.dt-dt.GOOD": {
    #         ">": 0
    #     },
    # },
    #                         ignore_filter_transformation=True)

    datasets = get_datasets(
        filter={
            "and": [
                {
                    "run_number": {
                        "and": [
                            {">": VALID_RUN_RANGE_START},
                            {"<": VALID_RUN_RANGE_STOP},
                        ]
                    }
                }
            ],
            "name": {"and": [{"<>": "online"}]},
            "dataset_attributes.global_state": {
                "and": [{"or": [{"=": "OPEN"}, {"=": "SIGNOFF"}, {"=": "COMPLETED"}]}]
            },
        },
        ignore_filter_transformation=True,
    )
    assert len(datasets) > 0


# test_get_datasets_with_ignore_filter()

# test_get_runs_with_ignore_filter()


def test_get_runs_not_compressed():
    runs = get_runs(
        filter={
            "run_number": {
                "and": [{">": VALID_RUN_RANGE_START}, {"<": VALID_RUN_RANGE_STOP}]
            },
            "dt-dt": "GOOD",
        },
        compress_attributes=False,
    )
    assert len(runs) > 0


def get_runs_with_combined_filter():
    runs = get_runs(
        filter={
            "run_number": {
                "and": [{">": VALID_RUN_RANGE_START}, {"<": VALID_RUN_RANGE_STOP}]
                # },
                # 'hlt_key': {
                #     'like': '%commissioning2018%'
                # },
                # 'significant': {
                #     '=': True
            }
        }
    )
    assert len(runs) > 0


def test_get_dataset_names_of_run():
    dataset_names = get_dataset_names_of_run(run_number=VALID_RUN_NUMBER)
    assert len(dataset_names) > 0


def test_get_dataset():
    dataset = get_dataset(run_number=VALID_RUN_NUMBER, dataset_name=VALID_DATASET_NAME)
    assert dataset["run_number"] == VALID_RUN_NUMBER
    assert dataset["name"] == VALID_DATASET_NAME
    dataset = get_dataset(
        run_number=INVALID_RUN_NUMBER, dataset_name=INVALID_DATASET_NAME
    )
    assert not dataset


def test_get_datasets():
    datasets = get_datasets(
        filter={
            "run_number": {
                "and": [{">": VALID_RUN_RANGE_START}, {"<": VALID_RUN_RANGE_STOP}]
            }
        }
    )
    assert len(datasets) > 0


def test_get_lumisections():
    lumisections = get_lumisections(VALID_RUN_NUMBER, VALID_DATASET_NAME)
    assert len(lumisections) > 0


def test_get_oms_lumisections():
    lumisections = get_oms_lumisections(VALID_RUN_NUMBER)
    assert len(lumisections) > 0
    dataset_lumisections = get_oms_lumisections(VALID_RUN_NUMBER, VALID_DATASET_NAME)
    assert len(dataset_lumisections) > 0


def test_get_lumisection_ranges():
    lumisections = get_lumisection_ranges(VALID_RUN_NUMBER, VALID_DATASET_NAME)
    assert len(lumisections) > 0


def test_get_oms_lumisection_ranges():
    lumisections = get_oms_lumisection_ranges(VALID_RUN_NUMBER)
    assert len(lumisections) > 0


def test_get_joint_lumisection_ranges():
    lumisections = get_joint_lumisection_ranges(VALID_RUN_NUMBER, VALID_DATASET_NAME)
    assert len(lumisections) > 0


def test_get_collisions18():
    runs = get_runs(filter={"class": "Collisions18"})
    assert len(runs) > 0


def test_get_or_run():
    runs = get_runs(filter={"run_number": {"or": [VALID_RUN_NUMBER]}})


def test_get_datasets_with_filter():
    datasets = get_datasets(
        filter={
            "run_number": {
                "and": [{">": VALID_RUN_RANGE_START}, {"<": VALID_RUN_RANGE_STOP}]
            },
            "tracker-strip": "GOOD",
        }
    )
    assert len(datasets) > 0


def test_generate_json():
    # https://docs.python.org/3/library/warnings.html#testing-warnings
    with warnings.catch_warnings(record=True) as w:
        warnings.simplefilter("always")
        generate_json(
            """
{
    "and": [
        {
            "or": [
                {
                    "==": [
                        {
                            "var": "dataset.name"
                        },
                        "/PromptReco/Collisions2018A/DQM"
                    ]
                }
            ]
        }
    ]
}
        """
        )
        assert len(w) == 1
        assert issubclass(w[-1].category, PendingDeprecationWarning)
        assert "deprecated" in str(w[-1].message)


# UNSAFE:
# def test_generate_json():
#     json_logic = """
#         {
#         "and": [
#             {
#                 "or": [
#                     {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018A/DQM"]},
#                     {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018B/DQM"]},
#                     {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018C/DQM"]},
#                     {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018D/DQM"]},
#                     {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018E/DQM"]},
#                     {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018F/DQM"]},
#                     {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018G/DQM"]},
#                     {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018H/DQM"]},
#                     {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018I/DQM"]}
#                 ]
#             },
#             { ">=": [{ "var": "run.oms.energy" }, 6000] },
#             { "<=": [{ "var": "run.oms.energy" }, 7000] },
#             { ">=": [{ "var": "run.oms.b_field" }, 3.7] },
#             { "in": [ "25ns", { "var": "run.oms.injection_scheme" }] },
#             { "==": [{ "in": [ "WMass", { "var": "run.oms.hlt_key" }] }, false] },

#             { "==": [{ "var": "lumisection.rr.dt-dt" }, "GOOD"] },
#             { "==": [{ "var": "lumisection.rr.csc-csc" }, "GOOD"] },
#             { "==": [{ "var": "lumisection.rr.l1t-l1tmu" }, "GOOD"] },
#             { "==": [{ "var": "lumisection.rr.l1t-l1tcalo" }, "GOOD"] },
#             { "==": [{ "var": "lumisection.rr.hlt-hlt" }, "GOOD"] },
#             { "==": [{ "var": "lumisection.rr.tracker-pixel" }, "GOOD"] },
#             { "==": [{ "var": "lumisection.rr.tracker-strip" }, "GOOD"] },
#             { "==": [{ "var": "lumisection.rr.tracker-track" }, "GOOD"] },
#             { "==": [{ "var": "lumisection.rr.ecal-ecal" }, "GOOD"] },
#             { "==": [{ "var": "lumisection.rr.ecal-es" }, "GOOD"] },
#             { "==": [{ "var": "lumisection.rr.hcal-hcal" }, "GOOD"] },
#             { "==": [{ "var": "lumisection.rr.muon-muon" }, "GOOD"] },
#             { "==": [{ "var": "lumisection.rr.jetmet-jetmet" }, "GOOD"] },
#             { "==": [{ "var": "lumisection.rr.lumi-lumi" }, "GOOD"] },
#             { "==": [{ "var": "lumisection.rr.dc-lowlumi" }, "BAD"] },

#             { "==": [{ "var": "lumisection.oms.cms_active" }, true] },
#             { "==": [{ "var": "lumisection.oms.bpix_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.fpix_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.tibtid_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.tecm_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.tecp_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.tob_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.ebm_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.ebp_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.eem_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.eep_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.esm_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.esp_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.hbhea_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.hbheb_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.hbhec_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.hf_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.ho_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.dtm_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.dtp_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.dt0_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.cscm_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.cscp_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.rpc_ready" }, true] },
#             { "==": [{ "var": "lumisection.oms.beam1_present" }, true] },
#             { "==": [{ "var": "lumisection.oms.beam2_present" }, true] },
#             { "==": [{ "var": "lumisection.oms.beam1_stable" }, true] },
#             { "==": [{ "var": "lumisection.oms.beam2_stable" }, true] }
#         ]
#     }
#     """
#     UNSAFE:
#     final_json = generate_json(json_logic)
#     assert final_json is not None
#     json_logic2 = {
#         "and": [
#             {
#                 "or": [
#                     {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018A/DQM"]},
#                     {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018B/DQM"]},
#                     {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018C/DQM"]},
#                     {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018D/DQM"]},
#                     {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018E/DQM"]},
#                     {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018F/DQM"]},
#                     {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018G/DQM"]},
#                     {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018H/DQM"]},
#                     {"==": [{"var": "dataset.name"}, "/PromptReco/Collisions2018I/DQM"]}
#                 ]
#             },
#             { ">=": [{ "var": "run.oms.energy" }, 6000] },
#             { "<=": [{ "var": "run.oms.energy" }, 7000] },
#             { ">=": [{ "var": "run.oms.b_field" }, 3.7] },
#             { "in": [ "25ns", { "var": "run.oms.injection_scheme" }] },
#             { "==": [{ "in": [ "WMass", { "var": "run.oms.hlt_key" }] }, False] },

#             { "==": [{ "var": "lumisection.rr.dt-dt" }, "GOOD"] },
#             { "==": [{ "var": "lumisection.rr.csc-csc" }, "GOOD"] },
#             { "==": [{ "var": "lumisection.rr.l1t-l1tmu" }, "GOOD"] },
#             { "==": [{ "var": "lumisection.rr.l1t-l1tcalo" }, "GOOD"] },
#             { "==": [{ "var": "lumisection.rr.hlt-hlt" }, "GOOD"] },

#             { "==": [{ "var": "lumisection.oms.bpix_ready" }, True] }
#         ]
#     }
#     final_json2 = generate_json(json_logic2)

#     assert final_json2 is not None

json_logic = {
    "and": [
        {">=": [{"var": "run.oms.energy"}, 6000]},
        {"<=": [{"var": "run.oms.energy"}, 7000]},
        {">=": [{"var": "run.oms.b_field"}, 3.7]},
        {"in": ["25ns", {"var": "run.oms.injection_scheme"}]},
        {"==": [{"in": ["WMass", {"var": "run.oms.hlt_key"}]}, False]},
        {"==": [{"var": "lumisection.rr.dt-dt"}, "GOOD"]},
        {"==": [{"var": "lumisection.rr.csc-csc"}, "GOOD"]},
        {"==": [{"var": "lumisection.rr.l1t-l1tmu"}, "GOOD"]},
        {"==": [{"var": "lumisection.rr.l1t-l1tcalo"}, "GOOD"]},
        {"==": [{"var": "lumisection.rr.hlt-hlt"}, "GOOD"]},
        {"==": [{"var": "lumisection.oms.bpix_ready"}, True]},
    ]
}


def test_create_json():
    json = create_json(
        json_logic=json_logic, dataset_name_filter="/PromptReco/Collisions2018A/DQM"
    )
    print(json)


def test_custom_filter():
    filter_arg = {
        "dataset_name": {"like": "%/PromptReco/Cosmics18CRUZET%"},
        "run_number": {
            "and": [{">=": VALID_RUN_RANGE_START}, {"<=": VALID_RUN_RANGE_STOP}]
        },
        "class": {"like": "Cosmics18CRUZET"},
        "global_state": {"like": "COMPLETED"},
        "ecal-ecal": "EXCLUDED",
    }

    datasets = get_datasets(filter=filter_arg)
    assert datasets
